//+-----------------------------------------------------------------------------------------------------------------------------+
//| Include Custom Functions Liabraries
//+-----------------------------------------------------------------------------------------------------------------------------+
//#include <AuraPro\Base\CDefines.mqh>
//#include <AuraPro\Libraries\GetFxSession.mqh>
//#include <AuraPro\Libraries\IsNewBar.mqh>

enum APPLIED_PRICE {
	// MQL ENUM_APPLIED_PRICE values
  price_CLOSE,
  price_OPEN,
  price_HIGH,
  price_LOW,
  price_MEDIAN,
  price_TYPICAL,
  price_WEIGHTED,
  price_AVERAGE,
	// ADDED CUSTOM HEIKEN ASHI PRICE values
  priceHA_CLOSE,
  priceHA_OPEN,
  priceHA_HIGH,
  priceHA_LOW,
  priceHA_MEDIAN,
  priceHA_TYPICAL,
  priceHA_WEIGHTED,
  priceHA_AVERAGE
};

struct SParam_iVWMA_TX {													// Note: we shall use either ModeTX or PeriodTX
	int									periodSlow;									// SLOW VWMA AVERAGING PERIOD
	int									periodMid;									// MID  VWMA AVERAGING PERIOD
	int									periodFast;									// FAST VWMA AVERAGING PERIOD
	APPLIED_PRICE				appliedPrice;								// VWMA APPLIED PRICE
	ENUM_MA_METHOD			maMethod;							    	// VWMA APPLIED MA METHOD
};
//+-----------------------------------------------------------------------------------------------------------------------------+
//| CLASS:				CBase.mqh
//| APPLICATION:	BASE CLASS FOR CUSTOM AND TECHNICAL INDICATORS
//+-----------------------------------------------------------------------------------------------------------------------------+
class CBase {

public:

		// PARAMETRIC CONSTRUCTOR TO INITIALISE CLASS
		CBase(string pSymbol,ENUM_TIMEFRAMES pTimeFrame);
	 ~CBase() {	}

protected:
		string								mSymbol;																// SET WHEN INITIALIZE CLASS
		int										mDigits;
		ENUM_TIMEFRAMES				mTimeFrame;															// SET WHILE USING IN DERIVED CLASS TO SET ITS PARAMETERS
		ENUM_APPLIED_VOLUME		mAppliedVolume;													// DEFAULT APPLIED VOLUME
		int										mShift;																	// HORIZONTAL BAR SHIFT
public:
		double 								mPriceHA[][4];
		// METHOD INTEDENDED TO BE CALLED FROM INDICATORS TO GET APPLIED PRICE AT SPECIFIED INDEX.
		// SYMBOL, TIME FRAME, MQL PRICE ARRAYS AND RATES_TOTAL PARAMETERS MUST BE PASSED.
		double 								GetIndexPrice(APPLIED_PRICE pAppliedPrice,const double &open[],const double &close[],
	                		 									const double &high[],const double &low[],int pIndex,int pRatesTotal);
	  int       						getPoints(double pPips) 					{ return(int)(pPips / SymbolInfoDouble(mSymbol,SYMBOL_POINT));   }
	  double    						getPips(int pPoints)    					{ return     (pPoints * SymbolInfoDouble(mSymbol,SYMBOL_POINT)); }
		string								dts(double pDouble)								{ return(DoubleToString(pDouble,mDigits)); }	// Price returned as string value

}; // END Of Class Definition
//+-----------------------------------------------------------------------------------------------------------------------------+
//| METHOD:				CBase()
//| APPLICATION:	PARAMETRIC CONSTRUCTOR
//+-----------------------------------------------------------------------------------------------------------------------------+
CBase::CBase(string pSymbol,ENUM_TIMEFRAMES pTimeFrame) {

		mSymbol 			 = pSymbol;
		mTimeFrame		 = pTimeFrame;
    mDigits 			 = (int)SymbolInfoInteger(mSymbol,SYMBOL_DIGITS);
		mAppliedVolume = VOLUME_TICK;																	// GetVolume() USES iVolumes() WHICH RETURNS ONLY TICK VOLUME
		mShift 				 = 0;

} // END Of method CBase()
//+-----------------------------------------------------------------------------------------------------------------------------+
//| METHOD:				GetIndexPrice()
//| APPLICATION:	METHOD TO RETURN SPECIFIED APPLIED PRICE FOR SPECIFIED INDEX
//| NOTE:					METHOD INTEDENDED TO BE CALLED FROM INDICATORS TO GET APPLIED PRICE AT SPECIFIED INDEX.
//+-----------------------------------------------------------------------------------------------------------------------------+
double CBase::GetIndexPrice(APPLIED_PRICE pAppliedPrice,const double &open[],const double &close[],const double &high[],
														const double &low[],int pIndex,int pRatesTotal) {

	int k = pIndex;
	//+---------------------------------------------------------------------------------------------------------------------------+
	//| LOOP TO CALCULATE HEIKEN ASHI CANDLE PRICES
	//+---------------------------------------------------------------------------------------------------------------------------+
		if(pAppliedPrice >= priceHA_CLOSE && pAppliedPrice <= priceHA_AVERAGE) {

			// RESET, IF NEEDED mPriceHA (Two Dimensional Array) ARRAY SIZE
	    if(ArrayRange(mPriceHA,0) != pRatesTotal)
	    	ArrayResize(mPriceHA,pRatesTotal);
	
			// CALCULATE HAOpen PRICE
			double HAOpen;
	    if(k > 0) {
	      HAOpen = (mPriceHA[k-1][2] + mPriceHA[k-1][3]) / 2.0;
	    }
	    else
	      HAOpen = open[k] + close[k];
	
			// CALCULATE HAClose, HAHigh AND HALow PRICE
	    double HAClose = (open[k] + high[k] + low[k] + close[k]) / 4.0;
	    double HAHigh  = MathMax(high[k],MathMax(HAOpen,HAClose));
	    double HALow   = MathMin(low[k],MathMin(HAOpen,HAClose));
	
			// SET FINAL HA CANDLE PRICES
	    if(HAOpen < HAClose) {
	      mPriceHA[k][0] = HALow;
	      mPriceHA[k][1] = HAHigh;
	    } 
	    else {
	      mPriceHA[k][0] = HAHigh;
	      mPriceHA[k][1] = HALow;
	    } 
	    mPriceHA[k][2] = HAOpen;
	    mPriceHA[k][3] = HAClose;
	
			switch(pAppliedPrice) {
			  case priceHA_CLOSE:     	return(HAClose);
			  case priceHA_OPEN:      	return(HAOpen);
			  case priceHA_HIGH:      	return(HAHigh);
			  case priceHA_LOW:       	return(HALow);
			  case priceHA_MEDIAN:   		return((HAHigh + HALow) / 2.0);
			  case priceHA_TYPICAL:   	return((HAHigh + HALow + HAClose) / 3.0);
			  case priceHA_WEIGHTED:  	return((HAHigh + HALow + HAClose + HAClose) / 4.0);
			  case priceHA_AVERAGE:   	return((HAHigh + HALow + HAClose + HAOpen)  / 4.0);
			}
		}
	//+---------------------------------------------------------------------------------------------------------------------------+
	//| LOOP TO CALCULATE MQL STANDARD PRICES
	//+---------------------------------------------------------------------------------------------------------------------------+
		else {
			switch(pAppliedPrice) {
				case price_CLOSE:					return(close[k]);
				case price_OPEN:					return(open[k]);
				case price_HIGH:					return(high[k]);
				case price_LOW:						return(low[k]);
			  case price_MEDIAN:    		return((open[k] + close[k]) / 2.0);
				case price_TYPICAL:				return((high[k] + low[k] + close[k]) / 3.0);
				case price_WEIGHTED:			return((high[k] + low[k] + close[k] + close[k]) / 4.0);
			}
		}

	return(0.00);						// if none of the Price Type above (warning signal!!! to be provided)

} // END Of method GetIndexPrice()
//+-----------------------------------------------------------------------------------------------------------------------------+
