#include  "CBase.mqh"
CBase			*cBase;
//+-----------------------------------------------------------------------------------------------------------------------------+
//| iVWMA.mq5
//| CUSTOM VOLUME WEIGHTED MOVING AVERAGE
//+-----------------------------------------------------------------------------------------------------------------------------+
  #property description 				" "
  #property description 				"Volume Weighted Moving Average Indicator"
  #property description 				"1) Selectable Price Method (including Heiken Ashi Price)"
  #property description 				"2) Selectable Moving Average Method"
  #include <MovingAverages.mqh>

  #property indicator_chart_window
  #property indicator_buffers  	7
  #property indicator_plots    	1

	#define		_BufferVWMA					0
	#define		_BufferMA						1
	#define		_BufferPriceVW			2
	#define		_BufferPriceVWMA		3
	#define		_BufferVolume 			4
	#define		_BufferVolumeMA			5
	#define		_BufferPrice				6
//+-----------------------------------------------------------------------------------------------------------------------------+
//| LOAD INPUT PARAMETERS
//| WE SUPPLY open[], high[], low[] AND close[] PRICE BUFFERS FROM THE INDICATOR, NO NEED TO INPUT Symbol / mTimeFrame HERE
//+-----------------------------------------------------------------------------------------------------------------------------+
  input 	int                 	MAPeriod      		= 200;								// MA Calculation Period [200]
  input 	APPLIED_PRICE  				AppliedPrice  		= priceHA_WEIGHTED;  	// APPLIED PRICE [price_CLOSE]
	input 	ENUM_MA_METHOD				AppliedMethod 		= MODE_LWMA;					// APPLIED MA METHOD [MODE_SMMA]
	input 	ENUM_APPLIED_VOLUME		AppliedVolume			= VOLUME_TICK;				// APPLIED VOLUME TYPE
	sinput 	color									AppliedColorVWMA	= clrGoldenrod;				// COLOR OF VWMA LINE
//+-----------------------------------------------------------------------------------------------------------------------------+
//| Define Indicator Handle(s) and Buffer(s)
//+-----------------------------------------------------------------------------------------------------------------------------+
	// BUFFER INDICATOR PLOTS
  double 					BufferVWMA[];
	// BUFFER MA / VWMA
  double 					BufferMA[];
	double					BufferPriceVW[];
	double					BufferPriceVWMA[];
	double					BufferVolume[];
	double					BufferVolumeMA[];
	// BUFFER MA
	double					BufferPrice[];
//+-----------------------------------------------------------------------------------------------------------------------------+
//| Custom indicator initialization function
//+-----------------------------------------------------------------------------------------------------------------------------+
int OnInit() {

    IndicatorSetString(INDICATOR_SHORTNAME,"iVWMA ("+IntegerToString(MAPeriod)+","+EnumToString(AppliedPrice)+","+
    									 EnumToString(AppliedMethod)+") ");
    IndicatorSetInteger(INDICATOR_DIGITS,_Digits);
  //+---------------------------------------------------------------------------------------------------------------------------+
  //| Indicator Buffers mapping / binding
  //+---------------------------------------------------------------------------------------------------------------------------+
    SetIndexBuffer(_BufferVWMA,BufferVWMA,INDICATOR_DATA);

    SetIndexBuffer(_BufferMA,BufferMA,INDICATOR_CALCULATIONS);
    SetIndexBuffer(_BufferPriceVW,BufferPriceVW,INDICATOR_CALCULATIONS);
    SetIndexBuffer(_BufferPriceVWMA,BufferPriceVWMA,INDICATOR_CALCULATIONS);
    SetIndexBuffer(_BufferVolume,BufferVolume,INDICATOR_CALCULATIONS);
    SetIndexBuffer(_BufferVolumeMA,BufferVolumeMA,INDICATOR_CALCULATIONS);

    SetIndexBuffer(_BufferPrice,BufferPrice,INDICATOR_CALCULATIONS);
  //+---------------------------------------------------------------------------------------------------------------------------+
  //| Plot Indicator Labels in DataWindow
  //+---------------------------------------------------------------------------------------------------------------------------+
    PlotIndexSetString(_BufferVWMA,PLOT_LABEL,"VW-"+StringSubstr(EnumToString(AppliedMethod),5,4)+" ("+IntegerToString(MAPeriod)+")");
    PlotIndexSetInteger(_BufferVWMA,PLOT_DRAW_TYPE,DRAW_LINE);
    PlotIndexSetInteger(_BufferVWMA,PLOT_LINE_STYLE,STYLE_SOLID);
    PlotIndexSetInteger(_BufferVWMA,PLOT_LINE_COLOR,AppliedColorVWMA);
    PlotIndexSetInteger(_BufferVWMA,PLOT_LINE_WIDTH,2);
    PlotIndexSetDouble(_BufferVWMA,PLOT_EMPTY_VALUE,EMPTY_VALUE);

		cBase = new CBase(_Symbol,_Period);

  //---
   return(INIT_SUCCEEDED);

} // End of OnInit()
//+-----------------------------------------------------------------------------------------------------------------------------+
//| INDICATOR Deinit()
//+-----------------------------------------------------------------------------------------------------------------------------+
void OnDeinit(const int reason) {

		delete cBase;

} // End of OnDeinit()
//+-----------------------------------------------------------------------------------------------------------------------------+
//| Custom indicator iteration function
//+-----------------------------------------------------------------------------------------------------------------------------+
int OnCalculate(const int         rates_total,
                const int         prev_calculated,
                const datetime    &time[],
                const double      &open[],
                const double      &high[],
                const double      &low[],
                const double      &close[],
                const long        &tick_volume[],
                const long        &volume[],
                const int         &spread[]) {

  // INDICATOR START TIME
  //ulong start = GetMicrosecondCount();
  //+---------------------------------------------------------------------------------------------------------------------------+
  //| POPULATE APPLIED PRICE DATA BUFFER
  //+---------------------------------------------------------------------------------------------------------------------------+
  	// CHECK ENOUGH BARS AVAILABLE
    if(rates_total < (2 * MAPeriod - 2))                			return(0);

  	// SET BAR START VALUE
    int vStart;
    if(prev_calculated == 0)	vStart = 0;  										// START POPULATING DATA BUFFER FROM Index[0]
    else 											vStart = (prev_calculated-1);		// SET START BAR EQUAL TO THE LAST INDEX IN BUFFER ARRAY

    for(int i = vStart; i < rates_total; i++) {

  		// calculate applied price for index[i]
			double Price = cBase.GetIndexPrice(AppliedPrice,open,close,high,low,i,rates_total);

			// calculate Volume Weighted Price and applicable volume
			BufferPriceVW[i] = (AppliedVolume == VOLUME_TICK) ? (Price * (double)tick_volume[i]) : (Price * (double)volume[i]);
			BufferVolume[i]  = (AppliedVolume == VOLUME_TICK) ? (double)tick_volume[i]					 : (double)volume[i];
		}

		// POPULATE BufferMA WITH MOVING AVERAGE PRICES BASED ON MA METHOD
    int vBegin = 0;                       										// INDEX, STARTING FROM WHICH DATA FOR SMOOTHING AVAILABLE

		switch(AppliedMethod) {

			case MODE_LWMA:
		    LinearWeightedMAOnBuffer(rates_total,prev_calculated,vBegin,MAPeriod,BufferPrice,BufferMA);

		    LinearWeightedMAOnBuffer(rates_total,prev_calculated,vBegin,MAPeriod,BufferPriceVW,BufferPriceVWMA);
		    LinearWeightedMAOnBuffer(rates_total,prev_calculated,vBegin,MAPeriod,BufferVolume,BufferVolumeMA);
				break;

			case MODE_EMA:
		    ExponentialMAOnBuffer(rates_total,prev_calculated,vBegin,MAPeriod,BufferPrice,BufferMA);

		    ExponentialMAOnBuffer(rates_total,prev_calculated,vBegin,MAPeriod,BufferPriceVW,BufferPriceVWMA);
		    ExponentialMAOnBuffer(rates_total,prev_calculated,vBegin,MAPeriod,BufferVolume,BufferVolumeMA);
				break;

			case MODE_SMMA:
		    SmoothedMAOnBuffer(rates_total,prev_calculated,vBegin,MAPeriod,BufferPrice,BufferMA);

		    SmoothedMAOnBuffer(rates_total,prev_calculated,vBegin,MAPeriod,BufferPriceVW,BufferPriceVWMA);
		    SmoothedMAOnBuffer(rates_total,prev_calculated,vBegin,MAPeriod,BufferVolume,BufferVolumeMA);
				break;

			default:	// SIMPLE MOVING AVERAGE
		    SimpleMAOnBuffer(rates_total,prev_calculated,vBegin,MAPeriod,BufferPrice,BufferMA);

		    SimpleMAOnBuffer(rates_total,prev_calculated,vBegin,MAPeriod,BufferPriceVW,BufferPriceVWMA);
		    SimpleMAOnBuffer(rates_total,prev_calculated,vBegin,MAPeriod,BufferVolume,BufferVolumeMA);
				break;

		} // END Of switch() operator

  //+---------------------------------------------------------------------------------------------------------------------------+
	//| MAIN CALCULATION OF VOLUME WEIGHTED MOVING AVERAGE
  //+---------------------------------------------------------------------------------------------------------------------------+
	  for(int i = vStart; i < rates_total; i++) {
	    if(BufferVolumeMA[i] != 0)
	      BufferVWMA[i] = BufferPriceVWMA[i] / BufferVolumeMA[i];
	  }

  // INDICATOR END TIME AND INDICATOR CALCULATION TIME IN MILI SECONDS
	//ulong finish = GetMicrosecondCount();  
	//PrintFormat("%s in %s took %.1f ms",__FUNCTION__,__FILE__,(finish-start)/1000);

  // OnCalculate() DONE. RETURN NEW prev_calculated FOR NEXT ITIRATION
	return(rates_total);

} // END Of OnCalculate())
//+-----------------------------------------------------------------------------------------------------------------------------+
